"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const db_object_1 = __importDefault(require("./db-object"));
const NUMERIC_TYPES = {
    smallint: "Integer" /* Integer */,
    integer: "Integer" /* Integer */,
    bigint: "Integer" /* Integer */,
    smallserial: "Integer" /* Integer */,
    serial: "Integer" /* Integer */,
    bigserial: "Integer" /* Integer */,
    decimal: "Exact" /* Exact */,
    numeric: "Exact" /* Exact */,
    real: "Floating" /* Floating */,
    "double precision": "Floating" /* Floating */,
};
class Type extends db_object_1.default {
    /** @ignore */
    constructor(args) {
        super(args);
        this.oid = args.oid;
        this.arrayOid = args.arrayOid;
        this.classOid = args.classOid;
        this.schema = args.schema;
        this.internalName = args.internalName;
        this.shortName = args.shortName || args.name;
        this.category = args.category;
        this.hasLength = args.hasLength || false;
        this.hasScale = args.hasScale || false;
        this.hasPrecision = args.hasPrecision || false;
        this.numericType = NUMERIC_TYPES?.[args.name];
    }
    /**
     * Full name of the object with '.' notation including [[Schema]] name.
     *
     * @example
     * const fullName = type.fullName; // public.phone_number
     */
    get fullName() {
        return `${this.schema.name}.${this.name}`;
    }
}
exports.default = Type;
//# sourceMappingURL=type.js.map